﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using ProtoBuf;

namespace Utility
{
    [ProtoContract]
    public class DateRange
    {
        [ProtoMember(1)]
        private Date startDate = Date.MinValue;

        [ProtoMember(2)]
        private Date endDate = Date.MaxValue;

        public DateRange(Date start, Date end)
        {
            this.startDate = start;
            this.endDate = end;
        }

        public DateRange(string strStartDate, string strEndDate)
        {
            this.startDate = strStartDate.ToDate();
            this.endDate = strEndDate.ToDate();
        }

        public DateRange()
        {

        }

        public Date StartDate
        {
            get { return startDate; }
            set { startDate = value; }
        }

        public Date EndDate
        {
            get { return endDate; }
            set { endDate = value; }
        }

        public override string ToString()
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(startDate.ToString());
            builder.Append(",");
            builder.Append(endDate.ToString());
            return builder.ToString();
        }

        /// <summary>
        /// 玩家是否输入了日期范围
        /// </summary>
        /// <param name="strStartDate"></param>
        /// <param name="strEndDate"></param>
        /// <returns></returns>
        public static bool IsInputRange(string strStartDate, string strEndDate)
        {
            if(!strStartDate.IsDateEmpty())
            {
                return true;
            }

            if(!strEndDate.IsDateEmpty())
            {
                return true;
            }

            return false;
        }

        public static bool IsValidRange(string strStartDate, string strEndDate)
        {
            if(strStartDate.IsDateEmpty() || strEndDate.IsDateEmpty())
            {
                return false;
            }

            if (!strStartDate.IsValidYMD())
            {
                return false;
            }

            if (!strEndDate.IsValidYMD())
            {
                return false;
            }

            Date startDate = strStartDate.ToDate();
            Date endDate = strEndDate.ToDate();

            if (startDate > endDate)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// 日期范围是否超过35天
        /// </summary>
        /// <returns></returns>
        public bool IsMoreThan35Days()
        {
            if(startDate.AddDays(35) < endDate)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// 获得日期范围所有的日期
        /// </summary>
        /// <returns></returns>
        public List<Date> GetAllDates()
        {
            List<Date> lstResults = new List<Date>();
            for(var date = startDate; date <= endDate; date=date.AddDays(1))
            {
                lstResults.Add(date);
            }
            return lstResults;
        }

        public bool Contains(Date date)
        {
            if(date > endDate)
            {
                return false;
            }

            if(date < startDate)
            {
                return false;
            }

            return true;
        }
    }
}
